;------------------------------------------------------------------------------
;This function will the directory and the underlying directories if they do not exist
;input:
;    1:path

;output:
;       eax:0 error, none 0:success
;flags: Z: Error  z: success
;------------------------------------------------------------------------------
if used createAllDirectories
createAllDirectories:
xor eax,eax  ;zero: no flags
xchg peax,pdword [pesp+psize*1] ;exchange path with flags
xchg peax,pdword [pesp]         ;exchange return address with path
push peax                       ;push return address
;jmp createAllDirectoriesEx
end if
;------------------------------------------------------------------------------
;This function will the directory and the underlying directories if they do not exist
;input:
;    1:path
;    2:flags
;      0x1:path includes filename

;output:
;       eax:0 error, none 0:success
;flags: Z: Error  z: success
;------------------------------------------------------------------------------
if used createAllDirectoriesEx | used createAllDirectories
createAllDirectoriesEx:
call denter

mov pesi,pdword [pebp+psize*2] ;path


;get last char of path
push pesi
call strlenW
sub peax,2    ; - 1 character to get the last character, not the zero byte

if used is32
  cmp dword [pesi],0x005c005c ;'\\'
  jne isNotLongPath__createAllDirectoriesEx
  cmp dword [pesi+0x4],0x005c003F ;'?\'
  jne isNotLongPath__createAllDirectoriesEx
else
  mov rcx,0x5C003F005C005C
  cmp qword [pesi],rcx   ;'\\?\'
  jne isNotLongPath__createAllDirectoriesEx
end if

add pesi,4*2
sub peax,4*2

isNotLongPath__createAllDirectoriesEx:
mov pedx,peax ;RDX/EDX => size of string
shr pedx,1 ;/2
add pesi,peax

mov pedi,pesi ;last char of path
xor eax,eax
xor ecx,ecx

STD ;string instruction backward order


analysePath:
  lodsw; get character
  
  cmp word [pesi],'\'
  je __Slash
  cmp word [pesi],'/'
  jne __notSlash

  __Slash:
    push peax ;push character on stack
    push pedi ;push char position
    inc ecx ;count amount of pushed chars and pos
    xor eax,eax ;place zero char
  
  __notSlash:
  stosw
  dec pedx
jnz analysePath

cld ;string instruction normal order

mov pesi,pdword [pebp+psize*2] ;path

loopThroughDirectories:

  ;check if directory exists
  push pesi
  call isDirectory

  jnz directoryExists

    ;create directory
    push pesi
    call createDirectory
    jz errorCreatingDirectory

  directoryExists:
  test pdword [pebp+psize*3],0x1 ;path includes filename
  jnz pathIncludesFilename

    sub pecx,1
    jc endLoopThroughDirectories
    jmp pathDoesNotIncludeFilename

  pathIncludesFilename:

    dec pecx
    jnz pathDoesNotIncludeFilename

      pop pedi ;char position
      pop peax ;saved char
      stosw    ;restore char

    jmp endLoopThroughDirectories

  pathDoesNotIncludeFilename:
  pop pedi ;char position
  pop peax ;saved char
  stosw    ;restore char

  nextLoopThroughDirectories:
  jmp loopThroughDirectories

endLoopThroughDirectories:
xor eax,eax
inc eax
test eax,eax

errorCreatingDirectory:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will copy a file from base to target. If the size is >50MB or zero it will use ASYNC copy method
;input:
;    1:Base url
;    2:Target url
;    3:Optional size qword (or pointer to qword size in x86)
;    4:optional pointer to qword to add the size to that is copied so far
;------------------------------------------------------------------------------
if used copyFile
copyFile:
;For sync and asycn copy:
;ecx=>error code
;edx=>Base file handle
;ebx=>target file handle
;esi=>buffer1

;x64:
;r15=>pointer to qword to add the size to that is copied so far
;x86:
;ebp+psize*5=>pointer to qword to add the size to that is copied so far

;For asycn copy:
;edi=>buffer2
;x64:
;r12=>fileSizeToCopy
;r13 => AsynchronousStruct_read
;r14 => AsynchronousStruct_write
;x86:
;esp => fileSizeToCopy
;esp+0x8 => AsynchronousStruct_read
;esp+0xC => AsynchronousStruct_write

BUFFER_SIZE_COPY_FILE equ 1024*1024*100  ;50MB
call denter

xor edi,edi ;EDI => 0

if used is64
  mov r15,qword [rbp+psize*5] ;pointer to qword to add the size to that is copied so far
end if

if used is64
  mov rax,pdword [pebp+psize*4]
  test rax,rax
  jz useASyncCopy

  cmp rax,BUFFER_SIZE_COPY_FILE  ;50MB
else
  cmp pdword [pebp+psize*4],0
  jz useASyncCopy
  push 0                      ;HighDword
  push BUFFER_SIZE_COPY_FILE  ;LowDword

  push pesp          ;50MB
  push pdword [pebp+psize*4]
  call cmpCarryQword

  ;get qword from stack
  pop eax
  pop eax
end if
jc copyFileInOneStream

;------------------------------------

useASyncCopy:
;open base file
push 0x0 or 0x10 ;READ or ASYNCHRONOUS_IO
push pdword [pebp+psize*2] ;base file
call openFileHandle
mov pedx,peax     ;R/EDX => base file handle
inc peax
test peax,peax    ;was file opened successfull?
mov ecx,9         ;ecx => error coude=> could not open base file
je endCopyFile

;Get base file attributes
push pdword [pebp+psize*2] ;base file
call getFileAttributes

;open target file
push peax       ;Attributes
push 0x1 or 0x8 or 0x10 ;WRITE&READ & CREATE_ALWAYS or ASYNCHRONOUS_IO
push pdword [pebp+psize*3] ;target file
call openFileHandleEx
mov pebx,peax     ;R/EBX => target file handle
inc peax
test peax,peax    ;was file opened successfull?
mov ecx,10        ;ecx => error coude=> could not open target file for write
je endCopyFileCloseBase

;get file size of base file
if used is32
  ;create place on stack for fileSizeToCopy(qword), AsynchronousStruct1 and AsynchronousStruct2
  sub esp,0x10

  push pesp   ;Long long pointer
end if
push pedx   ;Base file handle
call getfilesizeLongLong
if used is64
  mov r12,rax    ;R12 => fileSizeToCopy
end if

;check if it is the same as the one in the parameters
if used is64
  cmp peax,pdword [pebp+psize*4] ;file size
else
  push pesp                      ;Long long struct
  push pdword [pebp+psize*4]     ;file size
  call cmpQword

  ;get Long Long from stack
  ;-pop eax
  ;-pop eax
end if
mov ecx,2  ;ecx => Err code => base file size does not match
jne endCopyFileCloseBaseAndTarget

;-if used is32
;-  ;create place on stack for fileSizeToCopy(qword), AsynchronousStruct_read and AsynchronousStruct_write
;-  sub esp,0x10
;-end if

;Create buffer1
push BUFFER_SIZE_COPY_FILE  ;100MB
call GetMemLocation
mov pesi,peax                 ;R/ESI => copy buffer 1

;create AsynchronousStruct_read
call createAsynchronousStruct
if used is64
  mov r13,rax ;R13 => AsynchronousStruct_read
else
  mov dword [esp+0x8],eax ;ESP+0x8 => AsynchronousStruct_read
end if

;create AsynchronousStruct_write
call createAsynchronousStruct
if used is64
  mov r14,rax ;R14 => AsynchronousStruct_write
else
  mov dword [esp+0xC],eax ;ESP+0xC => AsynchronousStruct_write
end if

;set AsynchronousStruct_write to 'end of file'(-1)
if used is64
  push -1
  push peax
  call setOffsetAsynchronousStruct
else
  push peax
  call getOffsetAsynchronousStruct
  ;set qword
  mov dword [eax],-1
  mov dword [eax+0x4],-1
end if

startCopy:
  test pedi,pedi
  jz nothingToWriteYet

    if used is64
      push r13             ;AsynchronousStruct_read
    else
      push dword [esp+0x8] ;AsynchronousStruct_read
    end if
    push pedx  ;base file handle
    call GetWrittenBytesAsynchronous

    if used is64
      mov r8,rax  ;R8 => size written to buffer
    else
      push eax     ;save size written to buffer on stack --> that's why everything on stack is +0x4
    end if

    if used is64
      push r13             ;AsynchronousStruct_read
    else
      push dword [esp+0x8+0x4] ;AsynchronousStruct_read
    end if
    call getOffsetAsynchronousStruct ;get offset in x64 and pointer to offset in x86
    if used is64

      add rax,r8 ;add size written to buffer to the offset

      ;set offset
      push rax
      push r13             ;AsynchronousStruct_read
      call setOffsetAsynchronousStruct
    else
      mov ecx,dword [esp] ;size written to buffer
      push eax            ;offset pointer
      call AddToQword

      pop eax  ;restore size written to buffer --> that's why everything on stack is +0x0
    end if

    if used is64
      push r14             ;AsynchronousStruct_write
      push r8              ;size to write from buffer
    else
      push dword [esp+0xC] ;AsynchronousStruct_write
      push peax     ;siez to write from buffer
    end if
    push pedi     ;buffer
    push pebx     ;R/EBX => target file handle
    call writefileEx

  nothingToWriteYet:
  if used is64             ;if fileSizeToCopy==0
    test r12,r12
  else
    cmp dword [esp],0      ;lower dword from qword
    jnz moreToReadFromFile
    cmp dword [esp+0x4],0  ;higher dword from qword
  end if
  jnz moreToReadFromFile

    if used is64             ;fileSizeToCopy=-1
      dec r12
    else
      mov dword [esp],-1      ;lower dword from qword = -1
      mov dword [esp+0x4],-1  ;higher dword from qword = -1
    end if

    jmp noMoreToRead

  moreToReadFromFile:
  ;check fileSizeToCopy< BUFFER_SIZE_COPY_FILE
  mov eax,BUFFER_SIZE_COPY_FILE
  if used is64
    sub r12,rax              ;fileSizeToCopy-buffersize
  else
    push eax
    lea eax, dword [esp+0x4] ;fileSizeToCopy
    push eax

    mov eax,dword [eax]
    lea eax, dword [esp+0x8] ;fileSizeToCopy
    mov eax,dword [eax+4]
    call SubStackToQword

    mov eax,BUFFER_SIZE_COPY_FILE
  end if
  jnc noBufferOverflowCopyFile

    if used is64
      mov eax,r12d
      xor r12d,r12d
    else
      mov eax,dword [esp]    ;lower dword of qword
      mov dword [esp],0      ;set lower dword to 0
      mov dword [esp+0x4],0  ;set higher dword to 0
    end if
    add eax,BUFFER_SIZE_COPY_FILE

  noBufferOverflowCopyFile:
  if used is64
    push r13             ;AsynchronousStruct_read
  else
    push dword [esp+0x8] ;AsynchronousStruct_read
  end if
  push peax   ;size to read
  push pesi  ;buffer 1
  push pedx  ;base file handle
  call readfileEx

  xchg pedi,pesi

  noMoreToRead:
  test pedi,pedi
  jz nothingToWriteToBeginWith

  if used is64
    push r14 ;AsynchronousStruct_write
    push r13 ;AsynchronousStruct_read
    mov rax,rsp
  else
    lea eax,dword [esp+0x8]
  end if

  push -1  ;time to wait
  push peax ;list with Asynchronous structers
  push 2   ;push amount of Asynchronous structers
  call WaitForAsynchronousStructs

  if used is64
    add rsp,psize*2
  end if

  test pesi,pesi
  jnz secondBufferIsCreated

    ;Create buffer2
    push 1024*1024*100  ;100MB
    call GetMemLocation
    mov pesi,peax                 ;R/ESI => copy buffer 2
    jmp canNowReadAndWrite

  secondBufferIsCreated:
    if used is64
      push r14             ;AsynchronousStruct_write
    else
      push dword [esp+0xC] ;AsynchronousStruct_write
    end if   
    push pebx  ;target file handle
    call GetWrittenBytesAsynchronous 

    if used is64
      test r15,r15
      jz sizeIsNotCounted

      lock add qword [r15],rax
    else
      cmp pdword [pebp+psize*5],0
      jz sizeIsNotCounted

      mov ecx,eax  ;size that was written to the file
      push pdword [pebp+psize*5]
      call AddToQword
    end if

    sizeIsNotCounted:

  canNowReadAndWrite:
  if used is64             ;if fileSizeToCopy==-1
    cmp r12,-1
  else
    cmp dword [esp],-1      ;lower dword from qword
    jne startCopy
    cmp dword [esp+0x4],-1  ;higher dword from qword
  end if
  jne startCopy

nothingToWriteToBeginWith:
  if used is64
    push r14             ;AsynchronousStruct_write
  else
    push dword [esp+0xC] ;AsynchronousStruct_write
  end if
  call removeAsynchronousStruct

  if used is64
    push r13             ;AsynchronousStruct_read
  else
    push dword [esp+0x8] ;AsynchronousStruct_read
  end if
  call removeAsynchronousStruct
  jmp SetTimeDateFile

;------------------------------------

copyFileInOneStream:
;open base file
push 0x0 ;READ
push pdword [pebp+psize*2] ;base file
call openFileHandle
mov pedx,peax     ;R/EDX => base file handle
inc peax
test peax,peax    ;was file opened successfull?
mov ecx,9         ;ecx => error coude=> could not open base file
je endCopyFile

;Get base file attributes
push pdword [pebp+psize*2] ;base file
call getFileAttributes

;open target file
push peax       ;Attributes
push 0x1 or 0x8 ;WRITE&READ & CREATE_ALWAYS
push pdword [pebp+psize*3] ;target file
call openFileHandleEx
mov pebx,peax     ;R/EBX => target file handle
inc peax
test peax,peax    ;was file opened successfull?
mov ecx,10        ;ecx => error coude=> could not open target file for write
je endCopyFileCloseBase

;get file size of base file
if used is32
  ;Long long
  push 0
  push 0

  push pesp   ;Long long pointer
end if
push pedx   ;Base file handle
call getfilesizeLongLong

;check if it is the same as the one in the parameters
if used is64
  cmp peax,pdword [pebp+psize*4] ;file size
else
  push pesp                      ;Long long struct
  push pdword [pebp+psize*4]     ;file size
  call cmpQword
end if
mov ecx,2  ;ecx => Err code => base file size does not match
jne endCopyFileCloseBaseAndTarget

if used is64
  push qword [rbp+psize*4]    ;file size
else
  mov eax,dword [ebp+psize*4]
  push dword [eax]            ;file size low word
end if
call GetMemLocation
mov pesi,peax                 ;R/ESI => copy buffer

;read from base file
if used is64
  push qword [rbp+psize*4]    ;file size
else
  mov eax,dword [ebp+psize*4] ;file size low word
  push dword [eax]
end if
push pesi   ; buffer
push pedx   ;Base file handle
call readfile
if used is64
  cmp rax,qword [rbp+psize*4]    ;file size
else
  mov ecx,dword [ebp+psize*4]    ;file size low word
  cmp dword [ecx],eax
end if
mov ecx,11  ; ecx => Errcode =>could not read from base file
jne endCopyFileReleaseBuffer

;write to target file
if used is64
  push qword [rbp+psize*4]    ;file size
else
  mov eax,dword [ebp+psize*4] ;file size low word
  push dword [eax]
end if
push pesi   ; buffer
push pebx   ;Target file handle
call writefile
if used is64
  cmp rax,qword [rbp+psize*4]    ;file size
else
  mov ecx,dword [ebp+psize*4]    ;file size low word
  cmp dword [ecx],eax
end if
mov ecx,12  ; ecx => Errcode =>could not write to target file
jne endCopyFileReleaseBuffer

if used is64
  test r15,r15
  jz sizeIsNotCountedSync

  mov rax,qword [rbp+psize*4] ;file size
  lock add qword [r15],rax
else
  cmp pdword [pebp+psize*5],0
  jz sizeIsNotCountedSync

  mov ecx,dword [ebp+psize*4] ;file size low word
  push pdword [pebp+psize*5]
  call AddToQword
end if
sizeIsNotCountedSync:

SetTimeDateFile:
;--------------------------------------------
;Set the Time/Date
;--------------------------------------------

;Get Last modified Time/Date from base file
if used is32
  ;FILETIME
  push 0
  push 0

  push pesp
end if
push pedx   ;Base file handle
call getRawLastWriteTime
if used is64
  mov r12,rax ;save lastWriteTime
end if

;Set Last modified Time/Date from base file
if used is64
  push peax ;FileTime
else
  push pesp ;FileTime
end if
push pebx   ;Target file handle
call setRawLastWriteTime

;--------------------------------------------
;Check if the Time/Date is modified corectly
;--------------------------------------------
;Get Last modified Time/Date from target file
if used is32
  ;FILETIME
  push 0
  push 0

  push pesp
end if
push pedx   ;Base file handle
call getRawLastWriteTime
if used is64
  cmp rax,r12 ; check if Last modified Time/Date are the same
else
  lea eax,dword [esp+8]          ;FILETIME base file
  push pesp                      ;FILETIME target file
  push eax                       ;check if Last modified Time/Date are the same
  call cmpQword
end if
mov ecx,13  ;Errcode => could not set target file datetime
jne endCopyFileReleaseBuffer2

;--------------------------------------------
;Check if the size is modified corectly
;--------------------------------------------
if used is32
  lea eax,dword [esp+8]
  push eax            ;Long long pointer
end if
push pedx   ;Base file handle
call getfilesizeLongLong
if used is64
  mov r12,rax ;save file size
end if

if used is32
  push esp            ;Long long pointer
end if
push pebx   ;Target file handle
call getfilesizeLongLong
if used is64
  cmp rax,r12 ; R12 =>check if size is the same
else
  lea eax,dword [esp+8]
  push pesp                      ;FILETIME target file
  push eax                       ;FILETIME base file
  call cmpQword                  ;check if size is the same
end if
mov ecx,14  ;Errcode => the size of the target file does not match the size of the base file after coping
jne endCopyFileReleaseBuffer2

;--------------------------------------------
;Check if the attributes are modified corectly
;--------------------------------------------

;Get base file attributes
push pdword [pebp+psize*2] ;base file
call getFileAttributes
if used is64
  ;turn of flags that cannot be set this way
  mov r12d,ATTRIBUTES_NOT_SET_BY_SetFileAttributes
  and r12d,eax
  xor r12d,eax ;R12 => attributes
else
  push eax     ;Pushg attributes on stack

  ;turn of flags that cannot be set this way
  mov eax,ATTRIBUTES_NOT_SET_BY_SetFileAttributes
  and eax,dword [esp]
  xor dword [esp],eax
end if

;Get target file attributes
push pdword [pebp+psize*3] ;target file
call getFileAttributes

if used is64
  ;turn of flags that cannot be set this way
  mov r13d,ATTRIBUTES_NOT_SET_BY_SetFileAttributes
  and r13d,eax
  xor eax,r13d

  cmp r12,rax         ;base file attributes != target file attributes
else
  push peax
  mov eax,ATTRIBUTES_NOT_SET_BY_SetFileAttributes
  and eax,dword [esp]
  xor dword [esp],eax
  pop eax

  cmp eax,dword [esp] ;base file attributes != target file attributes
  pop eax             ;pop base file attributes from stack
end if
mov ecx,17 ;ecx => Errcode = the attributes of the target file do not match the attributes of the base file after coping
jne endCopyFileReleaseBuffer2

;no error
xor ecx,ecx
inc ecx

endCopyFileReleaseBuffer2:
test pedi,pedi
jz endCopyFileReleaseBuffer

push pedi
call FreeMemLocation

endCopyFileReleaseBuffer:
push pesi
call FreeMemLocation

endCopyFileCloseBaseAndTarget:
push pebx
call closefile

endCopyFileCloseBase:
push pedx
call closefile

endCopyFile:
mov eax,ecx ;Set return code
call dleave
ret psize*4
end if