
;Copy OneOnOne 1.00
;Copyright (c) 2022, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;This function will Divide a qword with a dword in 32-bit mode
;input
;  1:pointer to long long
;  EBX:to divide with
;output:
;  EDX = rest from last divide
;  EBX = divider
;-----------------------------------------------------------------------------
if used is32 & used DivQword
  DivQword:
;  call denter
  push eax ;save eax
  push esi ;save esi

  mov esi,dword [esp+0xC] ;long long

  xor edx,edx ;clear EDX
  mov eax,dword [esi+0x4] ;higher dword from qword
  div ebx  ;divide the higher dword
  mov dword [esi+0x4],eax

  mov eax,dword [esi]
  div ebx  ;divide the lower dword
  mov dword [esi],eax

  endDivQword:
  pop esi ;restore esi
  pop eax ;restore eax
;  call dleave
  ret 0x4
end if

;------------------------------------------------------------------------------
;This function will multiply a qword with a dword in 32-bit mode
;input
;  1:pointer to long long
;  ECX:to multiply with
;-----------------------------------------------------------------------------
if used is32 & used MulQword
  MulQword:
  call denter

  mov esi,dword [ebp+0x8] ;long long

  mov eax,dword [esi+0x4]
  mul ecx  ;multiply the higher dword
  mov dword [esi+0x4],eax

  mov eax,dword [esi]
  xor edx,edx
  mul ecx  ;multiply the lower dword
  mov dword [esi],eax
  jnc endMulQword

  add dword [esi+0x4],edx  ;add the rest of the previous multiplication to the higher dword

  endMulQword:
  call dleave
  ret 0x4
end if

;------------------------------------------------------------------------------
;This function will add a dword to a qword in 32-bit mode
;input
;  1:pointer to long long
;  ECX:dword to add
;-----------------------------------------------------------------------------
if used is32 & used AddToQword
  AddToQword:
  push eax

  mov eax,dword [esp+0x8] ;long long
  add dword [eax],ecx      ;add dword to the lower dword
  adc dword [eax+0x4],0  ;add carry (this is one if previous add made an overflow)

  pop eax
  ret 0x4
end if

;------------------------------------------------------------------------------
;This function will add a dword to a qword in 32-bit mode
;input
;  1:pointer to long long
;  EDX:dword to add
;-----------------------------------------------------------------------------
if used is32 & used AddEDXToQword
  AddEDXToQword:
  push eax

  mov eax,dword [esp+0x8] ;long long
  add dword [eax],edx      ;add dword to the lower dword
  adc dword [eax+0x4],0  ;add carry (this is one if previous add made an overflow)

  pop eax
  ret 0x4
end if

;------------------------------------------------------------------------------
;This function will add a dword to a qword in 32-bit mode
;input
;  1:pointer to long long
;  ECX:pointer to long long to add
;-----------------------------------------------------------------------------
if used is32 & used AddQwordToQword
  AddQwordToQword:
  push eax
  push edx

  mov eax,dword [esp+0xC] ;long long

  mov edx,dword [ecx]     ;lower dword of the qword to add
  add dword [eax],edx     ;add lower dword to lower dword

  mov edx,dword [ecx+4]   ;higher dword of the qword to add
  adc dword [eax+4],edx   ;add higher dword to higer dword with cary

  pop edx
  pop eax
  ret 0x4
end if

;------------------------------------------------------------------------------
;This function will substract a dword of a qword in 32-bit mode
;input
;  1:pointer to long long
;  ECX:dword to sub
;-----------------------------------------------------------------------------
if used is32 & used SubToQword
  SubToQword:
  push eax

  mov eax,dword [esp+0x8] ;long long
  sub dword [eax],ecx     ;sub dword to the lower dword
  sbb dword [eax+0x4],0   ;sub carry from higher word (this is one if previous sub made an overflow)

  pop eax
  ret 0x4
end if

;------------------------------------------------------------------------------
;This function will substract a dword of a qword in 32-bit mode
;input
;  1:pointer to long long
;  2:dword to sub
;-----------------------------------------------------------------------------
if used is32 & used SubStackToQword
  SubStackToQword:
  push eax
  push ecx

  mov eax,dword [esp+0xC] ;long long
  mov ecx,dword [esp+0x10];dword to sub
  sub dword [eax],ecx     ;sub dword to the lower dword
  sbb dword [eax+0x4],0   ;sub carry from higher word (this is one if previous sub made an overflow)

  pop ecx
  pop eax
  ret 0x8
end if

;------------------------------------------------------------------------------
;This function will substract a qword of a qword in 32-bit mode
;input
;  1:pointer to long long
;  2:dword to long long to 1 with sub
;-----------------------------------------------------------------------------
if used is32 & used SubQwordFromQword
  SubQwordFromQword:
  push eax
  push ecx
  push edx

  mov eax,dword [esp+0x10] ;long long (1)
  mov ecx,dword [esp+0x14] ;long long (2)

  mov edx,dword [ecx] ;EDX => lower dword from long long (2)
  sub dword [eax],edx ;sub lower dword(2) from the lower dword (1)

  mov edx,dword [ecx+0x4] ;EDX => higher dword from long long (2)
  sbb dword [eax+0x4],edx ;sub higher dword(2) AND carry from the lower dword (1)

  pop edx
  pop ecx
  pop eax
  ret 0x8
end if

;------------------------------------------------------------------------------
;This function will substract a dword of a qword in 32-bit mode
;input
;  1:pointer to long long
;output: Z flag
;-----------------------------------------------------------------------------
if used is32 & used TestZeroQword
  TestZeroQword:
  push eax
  push ecx

  mov eax,dword [esp+0xC] ;long long
  mov ecx,dword [eax] ;low word
  test ecx,ecx
  jnz endTestZeroQword

  mov ecx,dword [eax+0x4] ;high word
  test ecx,ecx

endTestZeroQword:
  pop ecx
  pop eax
  ret 0x4
end if

;------------------------------------------------------------------------------
;This function will compair two qwords in 32-bit mode
;input
;  1:pointer to long long
;  2:dword to long long
;return: flag Z => equal, z => not equal
;-----------------------------------------------------------------------------
if used is32 & used cmpQword
  cmpQword:
  push eax
  push ecx
  push edx

  mov eax,dword [esp+0x10] ;long long
  mov ecx,dword [esp+0x14] ;long long

  ;compair lower dword
  mov edx,dword [eax]
  cmp edx,dword [ecx]
  jne endCmpQword

  ;compair higher dword
  mov edx,dword [eax+0x4]
  cmp edx,dword [ecx+0x4]

endCmpQword:
  pop edx
  pop ecx
  pop eax
  ret 0x8
end if

;------------------------------------------------------------------------------
;This function will check if long long (1) > long long (2)
;input
;  1:pointer to long long
;  2:dword to long long
;return: flag C => long long (1) < long long (2), c => long long (1) > long long (2)
;-----------------------------------------------------------------------------
if used is32 & used cmpCarryQword
  cmpCarryQword:
  push eax
  push ecx
  push edx

  mov eax,dword [esp+0x10] ;long long
  mov ecx,dword [esp+0x14] ;long long

  ;compair lower dword
  mov edx,dword [eax]
  sub edx,dword [ecx]

  ;compair higher dword
  mov edx,dword [eax+0x4]
  sbb edx,dword [ecx+0x4]

endcmpCarryQword:
  pop edx
  pop ecx
  pop eax
  ret 0x8
end if

;------------------------------------------------------------------------------
;This function will mov a qword to an other qword in 32-bit mode
;input
;  1:pointer destination to long long
;  2:pointer source to long long
;output: Z flag
;-----------------------------------------------------------------------------
if used is32 & used movQword
  movQword:
  push esi
  push edi

  mov edi,dword [esp+0xC] ;destination long long
  mov esi,dword [esp+0x10] ;source long long

  movsd ;copy low dword
  movsd ;copy high dword

  pop edi
  pop esi
  ret 0x8
end if

;------------------------------------------------------------------------------
;This function will Divide a d/word with a dword and results if the rest is zero
;input
;  1:dword/qword to divide
;  2:to devide with
;output:
;  EAX = rest from last divide
;  Z: no rest z:rest
;-----------------------------------------------------------------------------
isRest:
call denter

xor edx,edx
mov peax,pdword [pebp+psize*2] ;dword/qword to divide
mov pecx,pdword [pebp+psize*3] ;dword/qword to divide with
div pecx

mov peax,pedx
call dleave
test peax,peax
ret psize*2