;------------------------------------------------------------------------------
;This function will convert size of a file to a readable string
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;
;-----------------------------------------------------------------------------
if used convertBytesToString
  convertBytesToString:
  call denter

  mov pedi,pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  xor esi,esi

  if used is32
    ;copy Qword before div
    push  dword [pedi+4]
    push  dword [pedi]
    mov pedi,pesp
  end if

  if used is64

  xor rdx,rdx

  nextRound:
    mov rcx,rdi
    shr rcx,10 ;divide by 1024

    test rcx,rcx
    jz RoundedSize
    inc esi ;next size

    mov rdx,rdi
    mov rax,rcx
    shl rax,10
    xor rdx,rax

    mov rdi,rcx
  jmp nextRound
  else ;if used i32
    mov ebx,1024 ;to divide with
    xor edx,edx

  nextRound:
    cmp dword [pedi+4],0 ;check higher dword on zero
    jnz sizeIsAbove1024

    cmp dword [pedi],1024 ;check lower dword on <1024
    jbe RoundedSize

  sizeIsAbove1024:
    push pedi
    call DivQword
    inc esi
  jmp nextRound
  end if

  RoundedSize:

    mov eax,edx
    xor edx,edx
    mov ecx,100
    mul ecx

    shr eax,10 ;divide by 1024

    push peax

    mov ebx,10 ;base
    mov peax,pedi
    mov pedi,pdword [pebp+psize*3] ;output string
    if used is64
      call internalIntToStr
    else
      call internalIntQToStr
    end if


    pop peax

    test peax,peax
    jz noDecimalA


    push peax

    mov ax,','
    stosb

    pop peax
    push peax

    cmp peax,10
    JAE moreThanTenA

    mov ax,'0'
    stosb

    moreThanTenA:
    mov ebx,10 ;base
    pop peax
    call internalIntToStr

    noDecimalA:

    mov ecx,esi

    jecxz copyBytes

    mov eax,' kB'
    dec ecx
    jz copyDword

    mov eax,' MB'
    dec ecx
    jz copyDword

    mov eax,' GB'
    dec ecx
    jz copyDword

    mov eax,' TB'
    dec ecx
    jz copyDword

    mov eax,' PB'
    dec ecx
    jz copyDword

    mov eax,' EB'
    jmp copyDword

  copyBytes:
    mov eax,' byt'
    stosd ;copy ' byt'
    mov eax,'es'
    stosd ;copy ' es',0,0
    dec pedi
    jmp endConvertBytesToString

  copyDword:
    stosd ;copy

endConvertBytesToString:
  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will convert size of a file to a readable UNICODE string
;output: 5GB (5.000.000.000 bytes)
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;-----------------------------------------------------------------------------
if used neatSizeFormat
  neatSizeFormat:
  call denter

  push pdword [pebp+psize*3] ;output string
  push pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  call convertBytesToStringW
  mov pedi,peax
  sub pedi,2 ;minus zero word

  mov eax, 0x00280020 ;' ('
  stosd

  push pedi ;for FormatBytes

  mov peax,pdword [pebp+psize*2]  ;pointer to Qword
  mov pebx,10                     ;base
  if used is32
    call internalIntQToStrW
  else
    call internalIntToStrW
  end if

  ;--push pedi already pushed
  call FormatBytes

  __findEndStr:
    cmp word [pedi],0
    jz __foundEndStr

    add pedi,2
  jmp __findEndStr

  __foundEndStr:

  mov eax,')'
  stosd

  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will convert size of a file to a readable UTF-8/Asci string
;output: 5GB (5.000.000.000 bytes)
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;-----------------------------------------------------------------------------
if used neatSizeFormatA
  neatSizeFormatA:
  call denter

  push pdword [pebp+psize*3] ;output string
  push pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  call convertBytesToString
  mov pedi,peax
  dec pedi ;minus zero byte

  mov ax, ' ('
  stosw

  push pedi ;for FormatBytes

  mov peax,pdword [pebp+psize*2]  ;pointer to Qword
  mov pebx,10                     ;base
  if used is32
    call internalIntQToStr
  else
    call internalIntToStr
  end if

  xor al,al
  stosb
  dec pedi

  ;--push pedi already pushed
  call FormatBytesA

  __findEndStrA:
    cmp byte [pedi],0
    jz __foundEndStrA

    inc pedi
  jmp __findEndStrA

  __foundEndStrA:

  mov ax,')'
  stosw

  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will convert size of a file to a readable UNICODE string
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;
;-----------------------------------------------------------------------------
if used convertBytesToStringW
  convertBytesToStringW:
  call denter

  mov pedi,pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  xor esi,esi

  if used is32
    ;copy Qword before div
    push  dword [pedi+4]
    push  dword [pedi]
    mov pedi,pesp
  end if

  if used is64

  xor rdx,rdx

  nextRoundW:
    mov rcx,rdi
    shr rcx,10 ;divide by 1024

    test rcx,rcx
    jz RoundedSizeW
    inc esi ;next size

    mov rdx,rdi
    mov rax,rcx
    shl rax,10
    xor rdx,rax

    mov rdi,rcx

  jmp nextRoundW
  else ;if used i32
    mov ebx,1024 ;to divide with
    xor edx,edx

  nextRoundW:
    cmp dword [pedi+4],0 ;check higher dword on zero
    jnz sizeIsAbove1024W

    cmp dword [pedi],1024 ;check lower dword on <1024
    jbe RoundedSizeW

  sizeIsAbove1024W:
    push pedi
    call DivQword
    inc esi
  jmp nextRoundW
  end if

  RoundedSizeW:

    mov eax,edx
    xor edx,edx
    mov ecx,100
    mul ecx

    shr eax,10 ;divide by 1024

    push peax

    mov ebx,10 ;base
    mov peax,pedi
    mov pedi,pdword [pebp+psize*3] ;output string
    if used is64
      call internalIntToStrW
    else
      call internalIntQToStrW
    end if

    pop peax

    test peax,peax
    jz noDecimal

    push peax

    mov ax,','
    stosw

    pop peax
    push peax

    cmp peax,10
    JAE moreThanTen

    mov ax,'0'
    stosw

    moreThanTen:
    mov ebx,10 ;base
    pop peax
    call internalIntToStrW

    noDecimal:

    mov ecx,esi

    jecxz copyBytesW

    if used is32
      mov eax,0x006B0020 ;' k'
      mov edx,0x0062 ;'b'
    else
      mov rax,0x0062006B0020 ;' kB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x004D0020 ;' M'
      mov edx,0x0042 ;'B'
    else
      mov rax,0x0042004D0020 ;'MB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00470020 ;' G'
      ;mov edx,du 'B'
    else
      mov rax,0x004200470020 ;' GB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00540020 ;' T'
      ;mov edx,du 'B'
    else
      mov rax,0x004200540020 ;' TB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00500020 ;' P'
      ;mov edx,du 'B'
    else
      mov rax,0x004200500020 ;' PB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00450020 ;' E'
      ;mov edx,du 'B'
    else
      mov rax,0x004200450020 ;' EB'
    end if
    jmp copyTwoDwords

  copyBytesW:
    if used is32
      mov eax,0x00620020 ;' b'
      stosd ;copy ' b'
      mov eax,0x00740079 ;'yt'
      stosd ;copy 'yt'
      mov eax,0x00730065 ;'es'
      xor edx,edx
      jmp copyTwoDwords
    else
      mov rax,0x0074007900620020 ;' byt'
      stosq ;copy ' byt'
      mov eax,0x00730065 ;'es'
      stosd
      xor ax,ax
      stosw
      jmp endConvertBytesToStringW
    end if

  copyTwoDwords:
    if used is32
      stosd ;copy
      mov eax,edx
      stosd
    else
      stosq
    end if

endConvertBytesToStringW:
  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers and add " bytes" behind the srting
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatBytes
FormatBytes:
  call denter

  ;add dots
  push pdword [pebp+psize*2]
  call FormatNumber

  call pushBytes__FormatBytes

    du ' bytes',0

pushBytes__FormatBytes:
  push pdword [pebp+psize*2]
  call strcatW

ReturnFormatBytes:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers and add " bytes" behind the srting as ASCI/UTF-8
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatBytesA
FormatBytesA:
  call denter

  ;add dots
  push pdword [pebp+psize*2]
  call FormatNumberA

  call pushBytes__FormatBytesA

    db ' bytes',0

pushBytes__FormatBytesA:
  push pdword [pebp+psize*2]
  call strcat

ReturnFormatBytesA:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatNumber
FormatNumber:
  call denter

  sub pesp,0x40 ;create output string on stack
  mov pedi,pesp
  add pedi,0x37; 0x1B

  ;get the last character of the input string
  mov pesi,pdword [pebp+psize*2] ;input string
  push pesi
  call strlenW
  add pesi,peax

  ;(R/E)CX => amount characters in input string
  mov pecx,peax
  shr pecx,1

  xor pedx,pedx
  STD ;reversed direction of string instructions

  ;copy zero character
  lodsw
  stosw

FormatLoop:
  ;add dot if three characters have been copied
  cmp edx,3
  jne NoPointYet

    mov ax,'.'
    stosw
    xor edx,edx

  NoPointYet:
    ;copy character
    inc edx
    lodsw
    stosw

loop FormatLoop
  cld ;clear reversed direction of string instructions

  add pedi,2

  ;copy the formated string to the output
  push pedi
  push pdword [pebp+psize*2]
  call strcpyW


ReturnFormatNumbers:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers as ASCI/UTF-8
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatNumberA
FormatNumberA:
  call denter

  sub pesp,0x40 ;create output string on stack
  mov pedi,pesp
  add pedi,0x37; 0x1B

  ;get the last character of the input string
  mov pesi,pdword [pebp+psize*2] ;input string
  push pesi
  call strlen
  add pesi,peax

  ;(R/E)CX => amount characters in input string
  mov pecx,peax
  ;-shr pecx,1

  xor pedx,pedx
  STD ;reversed direction of string instructions

  ;copy zero character
  lodsb
  stosb

FormatLoopA:
  ;add dot if three characters have been copied
  cmp edx,3
  jne NoPointYetA

    mov al,'.'
    stosb
    xor edx,edx

  NoPointYetA:
    ;copy character
    inc edx
    lodsb
    stosb

loop FormatLoopA
  cld ;clear reversed direction of string instructions

  inc pedi

  ;copy the formated string to the output
  push pedi
  push pdword [pebp+psize*2]
  call strcpy


ReturnFormatNumbersA:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will escape characters if neccecery
;input:
;    1:source UTF-16 string

;output:
;    (R/E)AX => JS string or null if the string doesn't need to be escaped

;------------------------------------------------------------------------------
if used toJSString
toJSString:
call denter

mov pesi,pdword [pebp+psize*2] ;string
xor ecx,ecx   ;total count
xor edx,edx   ;special char count

findEscapeableChars:
lodsw

cmp ax,'\'
je addSpecialChar

cmp ax,'"'
je addSpecialChar

;cmp ax,"'"
;je addSpecialChar

cmp ax,0x0008 ;\b
je addSpecialChar

cmp ax,0x0009 ;\t ???
je addSpecialChar

;\v???

cmp ax,0x000A ;\n
je addSpecialChar

cmp ax,0x000C ;\f
je addSpecialChar

cmp ax,0x000D ;\r
jne nextFindEscapeableChars

addSpecialChar:

inc pedx

nextFindEscapeableChars:
inc pecx

ignoreSpecialChar:
test ax,ax
jnz findEscapeableChars

xor eax,eax
test pedx,pedx
jz endToJSString

;shl pedx,2 ;times 4
;shl pecx,1 ;times 2
;add pedx,pecx

add pedx,pecx
shl pedx,1 ;times 2

push pedx
call GetMemLocation
mov pedi,peax ;new JSstring
mov pebx,peax ;new JSstring

mov pesi,pdword [pebp+psize*2] ;string

;-----
escapeChars:
lodsw

cmp ax,'\'
je escapeChar

cmp ax,'"'
je escapeChar

;cmp ax,"'"
;je escapeChar

mov cx,'b'
cmp ax,0x0008 ;\b
je escapeCharWithNewChar

mov cx,'t'
cmp ax,0x0009 ;\t ???
je escapeCharWithNewChar

;\v???

mov cx,'n'
cmp ax,0x000A ;\n
je escapeCharWithNewChar

mov cx,'f'
cmp ax,0x000C ;\f
je escapeCharWithNewChar

mov cx,'r'
cmp ax,0x000D ;\r
jne nextEscapeChars

escapeCharWithNewChar:
mov ax,cx

escapeChar:

mov word [pedi],'\'
add pedi,2

nextEscapeChars:
stosw

ignorEscapeChar:
test ax,ax
jnz escapeChars

mov peax,pebx

endToJSString:
call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will escape characters in an ASSCI string and transform it to an UTF-16 string
;input:
;    1:source UTF-16 string

;output:
;    (R/E)AX => JS string

;------------------------------------------------------------------------------
if used toJSStringAW
toJSStringAW:
call denter

mov pesi,pdword [pebp+psize*2] ;string
xor ecx,ecx   ;total count
xor edx,edx   ;special char count

findEscapeableCharsA:
lodsb

cmp al,'\'
je addSpecialCharA

cmp al,'"'
je addSpecialCharA

;cmp al,"'"
;je addSpecialCharA

cmp al,0x0008 ;\b
je addSpecialCharA

cmp al,0x0009 ;\t ???
je addSpecialCharA

;\v???

cmp ax,0x000A ;\n
je addSpecialCharA

cmp al,0x000C ;\f
je addSpecialCharA

cmp al,0x000D ;\r
jne nextFindEscapeableCharsA

addSpecialCharA:

inc pedx

nextFindEscapeableCharsA:
inc pecx

ignorEscapeCharA:
test al,al
jnz findEscapeableCharsA

;shl pedx,2 ;times 4
;shl pecx,1 ;times 2
;add pedx,pecx

add pedx,pecx
shl pedx,1 ;times 2

push pedx
call GetMemLocation
mov pedi,peax ;new JSstring
mov pebx,peax ;new JSstring

mov pesi,pdword [pebp+psize*2] ;string

;-----
xor ax,ax
xor cx,cx
escapeCharsA:
lodsb

cmp al,'\'
je escapeCharA

cmp al,'"'
je escapeCharA

;cmp al,"'"
;je escapeCharA

mov cl,'b'
cmp al,0x0008 ;\b
je escapeCharWithNewCharA

mov cl,'t'
cmp al,0x0009 ;\t ???
je escapeCharWithNewCharA

;\v???

mov cl,'n'
cmp ax,0x000A ;\n
je escapeCharWithNewCharA

mov cl,'f'
cmp al,0x000C ;\f
je escapeCharWithNewCharA

mov cl,'r'
cmp al,0x000D ;\r
jne nextEscapeCharsA

escapeCharWithNewCharA:
mov al,cl

escapeCharA:

mov word [pedi],'\'
add pedi,2

nextEscapeCharsA:
stosw

ignorEscapeCharWithNewCharA:
test ax,ax
jnz escapeCharsA

mov peax,pebx

endToJSStringA:
call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function has the same calling instruction as CreateOneString (except that the flags cannot be set)
;It will make a JSON string from the input.
;  - First one is name of the value
;  - Second one is the value
;input:
;    1..*:List of strings and integers
;        {Type: 0x0=last item; 0x1=ASCII string; 0x2=Unicode string; 0x3=Decimal number; 0x31=Hexidecimal number;
;               0x4=Collection as array, 0x41=Collection as object
;               0x5=Pointer to long long that needs to formated to "GB (5.000.000.000 bytes)"
;               0x40000 format number as size; 0x100000=format number with dots,
;               0x200000=collection is an nummeric collection
;         Pointer or integer
;         if type==0x9 or 0x91 then extra parameters:
;           begin in collection
;           end in collection}
;         if type==0xA=Begin collection
;         if type==0xB then=end collection
;         if type==0xC then=File attributes
;         if type==0xD=FILETIME that needs to transformed to a string
;         if type==0xE=Boolean

;
;output: (R/E)AX: pointer to memory with new unicode JSON string
;

;------------------------------------------------------------------------------
if used CreateJSon
CreateJSon:
push pesi
push pedi
push pecx
push pedx
push pebx
if used is64
  push r8
end if


lea pesi,pdword [pesp+psize+(psize*6)] ;pointer to first argument
xor ecx,ecx

countArguments:
  lodsp
  test peax,peax
  jz doneCounting

  mov pedx,peax

  and pedx,0xFFFF    ;only get lower bytes
  cmp pedx,0x9
  jz isCollection
  cmp pedx,0x91
  jz isCollection
  ;-jnz isNotCollection

isNotCollection:

  xchg pdword [pesi],peax ;swap order
  mov pdword [pesi-psize],peax

jmp doneSwap

isCollection:
    xchg pdword [pesi+psize*2],peax
    mov pdword [pesi-psize],peax
    lodsp
    xchg pdword [pesi],peax
    mov pdword [pesi-psize],peax
    lodsp

doneSwap:
  inc ecx
  lodsp
jmp countArguments

doneCounting:
  mov pebx,pesi   ;Save the pointer to the last pushed argument
  sub pesi,psize*2

  push 0              ;last item for CreateOnString

  call pushEndJson    ;push json end '}'

      db '}',0

pushEndJson:
  push 0x1  ;ASCI string

  test ecx,ecx
  jz endLoopThroughJsonItems

xor edx,edx
std

loopThroughJsonItems:
  lodsp             ;Get type of current item in list
  and eax,0xFFFF    ;only get lower bytes
  mov edi,eax       ;EDI => type of current item in list

  lodsp             ;Get pointer of current item in list
  xchg pedi,peax     ;EDI<>EAX => type of current item in list<> pointer of item


  ;;zero is last item in list
  ;test edi,edi
  ;jz endLoopThroughJsonItems

cmp peax,0xA
jz dontPushNextJsonItem

  test edx,edx
  jz dontPushNextJsonItem

    test edx,1
    jz doPushNextJsonItem

    call pushJsonValue

      db ':',0

pushJsonValue:
    jmp pushNextJsonItem

    doPushNextJsonItem:
    call pushNextJsonItem

      db ',',0

pushNextJsonItem:
  push 0x1

dontPushNextJsonItem:
  inc edx

  test pdword [pesi+psize*2],0x80000 or 0x100000
  jnz __pushStringStart

  cmp eax,0x3
  je isNumberAsName

  cmp eax,0x4
  jne isNotNcumber

isNumberAsName:
    test edx,1
    jz __pushStringStart

    jmp dontPushStringStart

isNotNcumber:

  cmp eax,0x9
  je dontPushStringStart

  cmp eax,0x91
  je dontPushStringStart

  cmp eax,0xA
  je dontPushStringStart

  cmp eax,0xB
  je dontPushStringStart

  cmp eax,0xE  ;Boolean
  je dontPushStringStart


  __pushStringStart:
    call pushStringStart ;Push begin string for strings ans hexidecimal values

      db '"',0

   pushStringStart:
   push 0x1 ;is asci string



  dontPushStringStart:
  cld       ;string op's upwards

  ;is ASCI
  dec eax
  jz sendStringA

  ;is Unicode
  dec eax
  jz sendStringW

  ;is decimal number
  dec eax
  jz isDecimal

  ;is 64bitNumber
  dec eax
  if used is32
    jz is64Decimal
  else
    jz isDecimal
  end if

  ;is formated to "GB (5.000.000.000 bytes)"
  dec eax
  jz isNeatSizeFormat

  ;0xA = begin object
  cmp eax,0xA-5
  jz __pushBeginObj

  ;0xB = end object
  cmp eax,0xB-5
  jz __pushEndObj

  ;0xC = File attributes
  cmp eax,0xC-5
  jz isFileAttributes

  ;0xD = FileTime
  cmp eax,0xD-5
  jz isFileTime

  ;0xE = Boolean
  cmp eax,0xE-5
  jz isBoolean

  ;0x91 = collection as array
  cmp eax,0x91-5
  jz isCollArr

  ;0x31 = hexidecimal, check for the '1'
  cmp eax,0x31-5
  jz Hexdecimal

  cmp eax,0x41-5
  if used is32
    jz Hexdecimal64
  else
    jz Hexdecimal
  end if

  ;0x91 = is collection as object
  ;--cmp eax,0x91-4
  ;--jz isCollObj

  call beginCreatingObj  ;push '] '

  endObj:
    db '}',0

  isCollArr:
  call pushEndArray  ;push '] '

  endArr:
    db ']',0

pushEndArray:
beginCreatingObj:
  push 0x1 ;asci string

  xchg pecx,pdword [pesi]       ;current number in stack (beginning at len(list)) <> begin in collection
  xchg pedx,pdword [pesi-psize] ;current number in stack (beginning at 0) <> end in collection


  ;ecx may not be zero
  test pecx,pecx
  jnz ecxIsNotZero

  mov ecx,1

  ecxIsNotZero:
  ;0 is all the items in the list
  test pedx,pedx
  jne edxIsNotZero

    push pedi          ;Collection
    call CountItems    ;Get next item
    mov pedx,peax

  edxIsNotZero:
  cmp pecx,pedx
  ja noItemsToAdd

  test pedx,pedx
  jz noItemsToAdd


  loopThroughCollectionItems:

    test pedx,pedx
    jz isNotObject ;stop if next item does not exist

    ;get first Item
    ;--push pedx          ;itemId
    push pedi          ;Collection
    call CountItems    ;Get next item
    cmp peax,pedx
    jb isNotObject ;stop if next item does not exist

    push pedx       ;itemId
    push pedi       ;Collection

    test pdword [pesi+psize*2],0x200000
    jnz isDwordCollection

    call GetItem    ;Get next item
    ;-test peax,peax
    ;-jz noItemsToAdd ;stop if next item does not exist
    jmp isStringCollection

    isDwordCollection:
    call GetDwordItem

    isStringCollection:
    cmp pdword [pesp+psize],endObj
    je loopThroughCollectionFirstItem

    cmp pdword [pesp+psize],endArr
    je loopThroughCollectionFirstItem

    call pushnextItemInCollection ;push ','
      db ',',0

pushnextItemInCollection:
    push 0x1  ;ASCI string

loopThroughCollectionFirstItem:
    test pdword [pesi+psize*2],0x200000
    jnz isNumberNoQuoteNeeded

    call pushBeginQuote ;push '"'
      db '"',0

pushBeginQuote:
    push 0x1  ;ASCI string

isNumberNoQuoteNeeded:
    push peax          ;push string for function CreateOneString

    test pdword [pesi+psize*2],0x200000
    jz isStringToPush

    isDwordToPush:
    push 0x3  ;dword/qword
    jmp isNumberNoEndQuoteNeeded

    isStringToPush:
    push peax          ;push string for function toJSString
    call toJSString    ;convert to JSon string
    test peax,peax
    jz pushNormalStringInCollection

    mov pdword [pesp],peax ;change string with JSsring
    push 0x2 or 0x20000 ;Jsstring and poiner may be deleted afther use
    jmp pushedCollectionString

  pushNormalStringInCollection:
    push 0x2  ;UTF-16 string
    ;-jmp pushedCollectionString


pushedCollectionString:
    call pushEndQuote ;push '"'
      db '"',0

pushEndQuote:
    push 0x1  ;ASCI string

isNumberNoEndQuoteNeeded:
    mov peax,pdword [pesi+psize*2]
    and eax,0xFFFF    ;only get lower bytes
    cmp eax,0x9  ;check if type is object
    jne isNotObject

    call pushJsonBeginValueObj ;push '":'

      db '":',0

pushJsonBeginValueObj:
    push 0x1  ;ASCI string
    push pedx ;push item number
    push 0x3  ;is number

    call pushJsonBeginNameObj

      db '"',0

pushJsonBeginNameObj:
    push 0x1  ;ASCI string


isNotObject:
    dec pedx
    cmp pedx,pecx
    jnc loopThroughCollectionItems

noItemsToAdd:
  xchg pecx,pdword [pesi]       ;begin in collection <> current number in stack (beginning at len(list))
  xchg pedx,pdword [pesi-psize] ;end in collection <> current number in stack (beginning at 0)
  sub pesi,psize*2


  mov peax,pdword [pesi+psize*4]
  and eax,0xFFFF    ;only get lower bytes
  cmp eax,0x9  ;check if type is object
  jne isNotBeginOfObject

  call pushBeginObj  ;push '{'

    db '{',0

isNotBeginOfObject:
  call pushBeginArray  ;push '['

    db '[',0

pushBeginArray:
pushBeginObj:
    push 0x1  ;ASCI string
  jmp nextLoopThroughJsonItems

  if used is32
    is64Decimal:
      push pedi

      test edx,1
      jz pushNumberType

        mov pdword [pesp],nextLoopThroughJsonItems ;ret value
      jmp pushNumberType

    Hexdecimal64:
      push pedi ;push pointer to string

      push createEndString ;ret value
      jmp pushNumberType
  end if

  isDecimal:
  push pedi

  push createEndString ;ret value

  test edx,1
  jz pushNumberType

    mov pdword [pesp],nextLoopThroughJsonItems ;ret value
  jmp pushNumberType

  Hexdecimal:
    push pedi ;push pointer to string

    push createEndString ;ret value

  pushNumberType:
    mov peax,pdword [pesi+psize*2] ;original type
    xchg pdword [pesp],peax  ;is hexidecimal string
    test pdword [pesp],0x80000 or 0x100000
    jnz createEndString
    jmp peax

  isNeatSizeFormat:
    push pedi
    push 0x5
    jmp createEndString

  isFileTime:
    push pedi           ;push TimeDate
    call FileTimeToString

    push peax           ;push the converted string
    push 0x1 or 0x20000 ;is asci string and free pointer
    jmp createEndString

  isBoolean:
    call pushFalse

      db 'false',0

    pushFalse:
    test pedi,pedi
    jz isFalseBoolan

      add pesp,psize ;remove false from stack
      call pushTrue

        db 'true',0

      pushTrue:
    isFalseBoolan:
    push 0x1        ;is asci string and free pointer
    jmp nextLoopThroughJsonItems

  isFileAttributes:
    push pedi           ;push attributes
    push FileAttributes ;push names for the flags
    call getFlagsByName ;get Unicode string with attributes
    ;--call toJSStringAW  ;attributes don't have escapeable characters, dont need to convert to JSon string

    push peax          ;push the converted string
    push 0x2 or 0x20000  ;is unicode string and free pointer
    jmp createEndString

  __pushEndObj:
  xor edx,edx
  call pushStringEnd

    db '}',0

  __pushBeginObj:
  call pushStringEnd

    db '{',0

  sendStringA:
    push pedi ;push pointer to string
    call toJSStringAW  ;convert to JSon string

    push peax          ;push the converted string
    push 0x2 or 0x20000  ;is asci string and free pointer
    jmp createEndString

  sendStringW:
    push pedi          ;push pointer to string
    call toJSString    ;convert to JSon string
    test peax,peax
    jz pushNormalString

    push peax          ;push the converted string
    push 0x2 or 0x20000  ;is unicode string and free pointer
    jmp createEndString

  pushNormalString:
    push pedi          ;push pointer to string
    push 0x2           ;is unicode string

    createEndString:
    call pushStringEnd ;Push begin string for strings ans hexidecimal values

      db '"',0

   pushStringEnd:
   push 0x1 ;is asci string

nextLoopThroughJsonItems:

   std       ;string op's upwards
   dec ecx
   jnz loopThroughJsonItems

cld

endLoopThroughJsonItems:
call callStartJSon  ;push json begin '{'

  db '{',0

callStartJSon:
push 0x1            ;is ASCI string

  call CreateOneString

mov pesi,pebx  ;restore the pointer to the last pushed argument

if used is64
  pop r8
end if
pop pebx
pop pedx
pop pecx
pop pedi

xchg pesp,pesi
push pdword [pesi+psize]  ;push return value
mov pesi,pdword [pesi]    ;push EDI that was saved earlier
ret
end if
;------------------------------------------------------------------------------
;This function combine names to the flags set and create a new string
;input:
;    1:list with flagnames. The list are string next to eachother, with a zero byte as separator
;    2:flags to print on screen

;
;output: (R/E)AX: pointer to memory with new unicode flags string
;

;------------------------------------------------------------------------------
if used getFlagsByName
getFlagsByName:
call denter

mov ecx,32
mov pesi,pdword [pebp+psize*2] ;flagNames
mov pebx,pdword [pebp+psize*3] ;flags

xor edx,edx ;count actuall set flags

push 0 ;last item

loopFeatures:
  shr ebx,1
  jnc nextBit ;jump if bit is not set

  test edx,edx
  jz dontPushSpace

  call pushSpace

    db ' ',0

pushSpace:
  push 0x1   ;ACSI
  
dontPushSpace:

  push pesi  ;ATTRIBUTE
  push 0x1   ;ACSI
  
  inc edx

nextBit:
  push pesi
  call strlen
  add pesi,peax
  inc pesi

  dec ecx
jnz loopFeatures

call CreateOneString

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function combine names to the flags set and create a new utf-8 string
;input:
;    1:list with flagnames. The list are string next to eachother, with a zero byte as separator
;    2:flags to print on screen
;    3:flags for CreateOneStringUtf8
;       0x1 The return value should be the size neccecery for this string
;    4:Buffer for CreateOneStringUtf8

;
;output: (R/E)AX: return value of CreateOneStringUtf8
;

;------------------------------------------------------------------------------
if used getFlagsByNameUtf8
getFlagsByNameUtf8:
call denter

mov ecx,32
mov pesi,pdword [pebp+psize*2] ;flagNames
mov pebx,pdword [pebp+psize*3] ;flags

xor edx,edx ;count actuall set flags

push 0 ;last item

loopFeatures__Utf8:
  shr ebx,1
  jnc nextBit__Utf8 ;jump if bit is not set

  test edx,edx
  jz dontPushSpace__Utf8

  call pushSpace__Utf8

    db ' ',0

pushSpace__Utf8:
  push 0x1   ;ACSI
  
dontPushSpace__Utf8:

  push pesi  ;ATTRIBUTE
  push 0x1   ;ACSI
  
  inc edx

nextBit__Utf8:
  push pesi
  call strlen
  add pesi,peax
  inc pesi

  dec ecx
jnz loopFeatures__Utf8
push pdword [pebp+psize*5] ;pointer
push pdword [pebp+psize*4] ;flags
call CreateOneStringUtf8

call dleave
ret psize*4
end if