
;Copy OneOnOne 1.00
;Copyright (c) 2022, Tycho Veltmeijer.
;All rights reserved.

PLUGIN_VERSION equ ProductVersionPrefix, ".", FileVersion

CPO_NAME_AUTHOR equ -1
CPO_NAME_VERSION equ -2

CPO_NAME_MAX_SIZE equ 04
CPO_NAME_MAX_TIMEOUT equ 05
CPO_NAME_MAX_SIZE_SEND equ 06

include 'include/function.inc'
include 'include/flags.inc'
include 'include/collection.inc'
include 'include_plugin_general/short_functions.inc'
include 'include/stringFunctions.inc'
include 'include/math.inc'
include 'include/loopDir.inc'
include 'include/compair.inc'
include 'include/timedate.inc'
include 'include/fileFunctions.inc'
include 'include_tvs_server_plugin_backup_system/plugin_handle_connection.inc'
include 'include/backup.inc'
include 'include/buffer.inc'

sizeSubportedProtocols equ endSubportedProtocols - subportedProtocols

include 'include_tvs_server_plugin_backup_system/plugin_bind.inc'
include 'include_plugin_general/__asm_exchange_tables_GetProtocolNames.inc'

mymod pointer ?

virtual at 0
call initHTTPProtocol
initHTTPProtocol:
end virtual

;------------------------------------------------------------------------------
;Main
;------------------------------------------------------------------------------
main:
StartProc 3
call denter

  cmp dword [pebp+psize*3],1 ;DLL_PROCESS_ATTACH
  jne dontInitDLL

    mov peax,pdword [pebp+psize*2] ;get HMODULE
    mov [mymod],peax

    call initGetSmallestPortionPageAllocation

    ;When action 'Start' is initiated, all pointers are released and replaced with new ones.
    ;Other actions may still use one of those pointers, therefor this collection is introduced.
    ;This is used as a lock, full lock if action 'start' is executed, write lock if any othjer action is called.
    ;;Start can only get a full lock if no other thread had write locked it, and therefore waits until they are finished.
    call CreateCollection
    mov [freeLock],peax

    call pushTVS_HTTP_PROTOCOL

      if used is32
        db 'tvs_server_http_protocol_x86.dll',0
      else
        db 'tvs_server_http_protocol_x64.dll',0
      end if

    pushTVS_HTTP_PROTOCOL:
    call loadmodule ;load HTTP protocol
    test peax,peax
    jz endMain

    mov pedx,peax
    xor ecx,ecx
    ;get pointers to functions
    mov esi,dword [pedx+0x3C] ;get offset of PE header
    cmp word [pedx+pesi+0x4+0x10],0x18  ;if SizeOfOptionalHeader<0x18 then BaseOfCode is not defined.
    cmovb eax,ecx
    jb endMain

    mov esi,dword [pedx+pesi+0x4+0x14+0x14] ;[Base + offset PE header + signature + fileheader + offset of BaseOfCode in optional header] = BaseOfCode
    test esi,esi
    cmovz eax,ecx
    jz endMain

    add pesi,pedx  ;pointer to first section with code

    ;check version dword
    lodsd ;get version
    cmp ax,1
    jz copyHTTPFunctions

    test ax,ax
    cmovnz eax,ecx
    jnz endMain

    copyHTTPFunctions:
    mov ecx,(endImportListHttp-beginImportListHttp)/psize
    mov  pedi,beginImportListHttp
    rep movsp ;copy the pointers

dontInitDLL:
  xor eax,eax
  inc eax

endMain:
EndProc 3
call dleave
ret (FunctionReturnFactor*3)

;------------------------------------------------------------------------------
;The StartProtocol function need to initiate the protocol named in the ProtocolSession structure.
;------------------------------------------------------------------------------
StartProtocol:
StartProc 1
call denter

call initGetSmallestPortionPageAllocation

;Get the protocolName member from ProtocolSession
mov peax,pdword [pebp+(psize*2)]
mov peax,pdword[peax+(psize*1)]

;Do we init manipulator?
push TVS_SOFTWARE_BACKUP_SYSTEM
push peax
call strcmp
test eax,eax
jnz endStartProtocol

;init manipulator
push V_TV_SERVER_102 ;version
push 4 ;extraSize
push 0 ;pSign
push PT_MANIPULATOR  ;type
push pdword [pebp+(psize*2)] ;psess
call [SetProtocolInformation]
test peax,peax
jz endStartProtocol

push httpControlPanelOptions  ;cpo
push pdword [pebp+(psize*2)] ;psess
call [SetControlPanelOptions]

push pdword [pebp+(psize*2)] ;psess
call [AcceptProtocol]

mov pecx,pdword [pebp+(psize*2)] ;psess
mov [psess_limit_connections],pecx
jmp endStartProtocol

;return
endStartProtocol:
call dleave
EndProc 1
ret (FunctionReturnFactor*1)