
;Copy OneOnOne 1.00
;Copyright (c) 2022, Tycho Veltmeijer.
;All rights reserved.


ATTRIBUTES_NOT_SET_BY_SetFileAttributes equ 0x10 or 0x40 or 0x200 or 0x400 or 0x800 or 0x4000

if used getMimeTypeFromExtension
valueNameContentType db 'Content Type',0

;------------------------------------------------------------------------------
;This function will get a MIM
;input:
;    (R/E)AX:pointer with extension
;output:
;    (R/E)AX=> pointer with mime-type
;    (R/E)CX=> size of pointer
;------------------------------------------------------------------------------
getMimeTypeFromExtension:
push pedx

push valueNameContentType  ;valueName
push 0                     ;SubSubSubSubKey
push 0                     ;SubSubSubKey
push 0                     ;SubSubKey
push peax                  ;file extension
mov eax,HKEY_CLASSES_ROOT
push peax                  ;hKey
call ReadRegEx
test edx,edx
jz endGetMimeTypeFromExtension

;ReadRegEx failed, clear allocated memory
push peax
call FreeMemLocation

xor eax,eax  ;return zero

endGetMimeTypeFromExtension:
pop pedx
ret
end if

;------------------------------------------------------------------------------
;This function will open a file
;input:
;    1:pointer with filename
;    2:flags:
;       0x0  READ
;       0x1  WRITE&READ
;       0x2  OPEN ALWAYS
;       0x4  SHARE_READ_WHILE_WRITE
;       0x8  CREATE_ALWAYS
;       0x10 ASYNCHRONOUS_IO (OVERLAPPED)

;output:
;    (R/E)AX=> filehandle or -1 on failure
;------------------------------------------------------------------------------
if used openFileHandle
openFileHandle:
mov eax,0x80
xchg peax,pdword [pesp+psize*2] ;exchange flags with zero (normal atributes)
xchg peax,pdword [pesp+psize*1] ;exchange filename with flags
xchg peax,pdword [pesp]         ;exchange return address with filename
push peax                       ;push return address
;jmp openFileHandleEx
end if

;------------------------------------------------------------------------------
;This function will open a file
;input:
;    1:pointer with filename
;    2:flags:
;       0x0  READ
;       0x1  WRITE&READ
;       0x2  OPEN ALWAYS
;       0x4  SHARE_READ_WHILE_WRITE
;       0x8  CREATE_ALWAYS
;       0x10 ASYNCHRONOUS_IO (OVERLAPPED)
;    3:attributes:
;       0x1 FILE_ATTRIBUTE_READONLY
;       0x2 FILE_ATTRIBUTE_HIDDEN
;       0x4 FILE_ATTRIBUTE_SYSTEM
;       0x20 FILE_ATTRIBUTE_ARCHIVE
;       0x80 FILE_ATTRIBUTE_NORMAL
;       0x100 FILE_ATTRIBUTE_TEMPORARY
;       0x1000 FILE_ATTRIBUTE_OFFLINE
;       0x4000 FILE_ATTRIBUTE_ENCRYPTED


;output:
;    (R/E)AX=> filehandle or -1 on failure
;------------------------------------------------------------------------------
if used openFileHandleEx | used openFileHandle
openFileHandleEx:
call denter
MakeStack

;add '\\?\' if nececerry
push pdword [pebp+psize*2]
call createLongPath
mov pebx,peax

mov pecx,pdword [pebp+psize*4] ;dwFlagsAndAttributes
mov edi,FILE_SHARE_READ ;dwShareMode

mov edx,0x80000000  ;GENERIC_READ
test pdword [pebp+psize*3],0x1 ;WRITE&READ
jz noWriteRightsRequired

  push pebx
  call getFileAttributes

  test eax,0x1 ;check if readonly
  jz notReadOnlyFile

    xor eax,0x1 ;remove readonly flag
    ;make sure we can write to the file, by setting the attributes
    push peax
    push pebx
    call setFileAttributes

  notReadOnlyFile:
  or edx,0x40000000 ;GENERIC_WRITE
  test pdword [pebp+psize*3],0x4 ;SHARE_READ_WHILE_WRITE
  jnz noWriteRightsRequired

  xor edi,edi ;dwShareMode

noWriteRightsRequired:
mov esi,OPEN_EXISTING ;ESI => parameter CreationDisposition
test pdword [pebp+psize*3],0x2 ;OPEN ALWAYS
jz noFileOpenAlways

  mov esi,OPEN_ALWAYS

noFileOpenAlways:
test pdword [pebp+psize*3],0x8 ;CREATE_ALWAYS
jz noFileCreateAlways

  mov esi,2; CREATE_ALWAYS

noFileCreateAlways:
test pdword [pebp+psize*3],0x10 ;ASYNCHRONOUS_IO
jz noAsynchronousIO

  or ecx,0x40000000  ;FILE_FLAG_OVERLAPPED

noAsynchronousIO:
argpush 7,0                     ;hTemplateFile
argpush 6,pecx                  ;dwFlagsAndAttributes
argpush 5,pesi                  ;dwCreationDisposition
argpush 4,0                     ;lpSecurityAttributes
argpush 3,pedi                  ;dwShareMode
argpush 2,edx                   ;dwDesiredAccess
argpush 1,pebx                  ;filename
call [CreateFileW]

cmp pebx,pdword [pebp+psize*2]
je dontFreePointer__openFileHandleEx

  push peax ;save return value

  push pebx
  call FreeMemLocation

  pop peax

dontFreePointer__openFileHandleEx:

call dleave
ret psize*3
end if

;------------------------------------------------------------------------------
;This function will delete a file
;input:
;    1:pointer with filename


;output:
;    (R/E)AX=>0:error 1:success
;------------------------------------------------------------------------------
if used removeFile
removeFile:
call denter
MakeStack

;add '\\?\' if nececerry
push pdword [pebp+psize*2]
call createLongPath
mov pebx,peax

push pebx
call getFileAttributes

test eax,0x1 ;check if readonly
jz notReadOnlyFile__removeFile

  xor eax,0x1 ;remove readonly flag
  ;make sure we can write to the file, by setting the attributes
  push peax
  push pebx
  call setFileAttributes

notReadOnlyFile__removeFile:
argpush 1,pebx
call [DeleteFileW]
test peax,peax
jz deleteFailed

  ;make sure it returns 1 and not >0
  xor eax,eax
  inc eax

deleteFailed:
cmp pebx,pdword [pebp+psize*2]
je dontFreePointer__removeFile

  push peax ;save return value

  push pebx
  call FreeMemLocation

  pop peax

dontFreePointer__removeFile:

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will return if the function openFileHandle had failed because of not finding the file/path
;input:
;    None

;output:
;    (R/E)AX=> 1: file was not found
;              0: other error
;------------------------------------------------------------------------------
if used doesFileNotExist
doesFileNotExist:
call denter
MakeStack

call [GetLastError]
mov ecx,eax
xor eax,eax

cmp ecx,0x2 ;ERROR_FILE_NOT_FOUND
je returnTrue

cmp ecx,0x3 ;ERROR_PATH_NOT_FOUND
jne endDoesFileNotExist

returnTrue:
inc eax

endDoesFileNotExist:
call dleave
ret
end if

;------------------------------------------------------------------------------
;This function will return the size of a file (with limit of 4GiB on x86)
;input:
;    1:file handle

;output:
;    (R/E)AX => low integer with size

;------------------------------------------------------------------------------
if used getfilesize | (used getfilesizeLongLong & used is64)
getfilesize:
if used getfilesizeLongLong & used is64
  getfilesizeLongLong:
end if
call denter
MakeStack

if used is32
  ;renember: we cant use GetFileSizeEx in x86 if we want to support Windows NT
  push 0                    ;high word is not needed here (call getfilesizeLongLong in x86 mode for long long support)
  push dword [pebp+psize*2] ;file handle
  call [GetFileSize]
else
  ;x64 is supported since XP and GetFileSizeEx is supported since 2000, so no problem here.
  lea rdx,qword [pesp+psize*3] ;free spot on the stack for the return long long
  mov rcx,qword [pebp+psize*2] ;file handle
  call [GetFileSizeEx]

  mov rax,qword [pesp+psize*3] ;set file size on RAX
end if

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will return the size of the file
;input x86:
;    1:file handle
;    2:pointer to long long
;input x64:
;    1:file handle

;output x86:
;    EAX => low integer with size
;    [2] => high integer with size

;output x64:
;    RAX => size of file
;------------------------------------------------------------------------------
if used getfilesizeLongLong & used is32
getfilesizeLongLong:
call denter

mov pedi,dword [ebp+psize*3]  ;long long
add pedi,4                   ;high part of the long long

push pedi                ;the return pointer for the higher part of the size
push dword [pebp+psize*2] ;file handle
call [GetFileSize]

sub pedi,4 ;low part of the long long
stosd

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will read out a file
;input:
;    1:file handle
;    2:pointer to buffer
;    3:size to read

;output:
;    EAX => size read
;------------------------------------------------------------------------------
if used readfile
readfile:
xor eax,eax
xchg peax,pdword [pesp+psize*3] ;exchange size to read with zero (no AsynchronousStruct)
xchg peax,pdword [pesp+psize*2] ;exchange buffer with size to read
xchg peax,pdword [pesp+psize*1] ;exchange handle with buffer
xchg peax,pdword [pesp]         ;exchange return address with handle
push peax                       ;push return address
;jmp readfileEx
end if

;------------------------------------------------------------------------------
;This function will read out a file
;input:
;    1:file handle
;    2:pointer to buffer
;    3:size to read
;    4:AsynchronousStruct

;output:
;    EAX => size read
;------------------------------------------------------------------------------
if used readfileEx | used readfile
readfileEx:
call denter
MakeStack

if used is32
  push 0 ;make space for the return read size
  mov pedx,pesp
else
  lea r9,dword [rsp+psize*6] ;argument nNumberOfBytesToRead => pointer to free space on stack
end if

mov peax,pdword [pebp+psize*5]
argpush 5,peax  ;lpOverlapped
if used is32
  push edx ;nNumberOfBytesToRead
else
  ;argpush 4,r9 (already done)
end if
argpush 3,pdword [pebp+psize*4] ;size to read
argpush 2,pdword [pebp+psize*3] ;buffer
argpush 1,pdword [pebp+psize*2] ;file handle
call [ReadFile]

if used is32
  pop peax ;pop the read size
else
  mov eax,dword [rsp+psize*6] ;the read size
end if

call dleave
ret psize*4
end if

;------------------------------------------------------------------------------
;This function will write to a file
;input:
;    1:file handle
;    2:pointer to buffer
;    3:size to write

;output:
;    EAX => size written
;------------------------------------------------------------------------------
if used writefile
writefile:
xor eax,eax
xchg peax,pdword [pesp+psize*3] ;exchange size to write with zero (no AsynchronousStruct)
xchg peax,pdword [pesp+psize*2] ;exchange buffer with size to write
xchg peax,pdword [pesp+psize*1] ;exchange handle with buffer
xchg peax,pdword [pesp]         ;exchange return address with handle
push peax                       ;push return address
;jmp writefileEx
end if

;------------------------------------------------------------------------------
;This function will write to a file
;input:
;    1:file handle
;    2:pointer to buffer
;    3:size to write
;    4:AsynchronousStruct

;output:
;    EAX => size written
;
;------------------------------------------------------------------------------
if used writefileEx | used writefile
writefileEx:
call denter
MakeStack

if used is32
  push 0 ;make space for the return write size
  mov pedx,pesp
else
  lea r9,dword [rsp+psize*6] ;argument lpNumberOfBytesWritten => pointer to free space on stack
end if

mov peax,pdword [pebp+psize*5]
argpush 5,peax  ;lpOverlapped
if used is32
  push edx ;nNumberOfBytesToRead
else
  ;argpush 4,r9 (already done)
end if
argpush 3,pdword [pebp+psize*4]  ;size to write
argpush 2,pdword [pebp+psize*3] ;buffer
argpush 1,pdword [pebp+psize*2] ;file handle
call [WriteFile]

if used is32
  pop peax ;pop the read size
else
  mov eax,dword [rsp+psize*6] ;the read size
end if

call dleave
ret psize*4
end if

;------------------------------------------------------------------------------
;This function will create an AsynchronousStruct for write operations with Asynchronous operations
;input:
;    none

;output:
;    EAX => pointer to AsynchronousStruct
;------------------------------------------------------------------------------
if used createAsynchronousStruct
createAsynchronousStruct:
push pecx

call createEvent
mov pecx,peax ;ECX = event

push (psize*2)+(4*2)+psize  ;size of OVERLAPPED
call GetMemLocation
mov pdword [peax+(psize*2)+(4*2)], pecx ;set hEvent

pop pecx
ret
end if

;------------------------------------------------------------------------------
;This function creates an event that can be used by Asynchronous operations
;input:
;    none

;output:
;    EAX => pointer to event
;------------------------------------------------------------------------------
if used createEvent
createEvent:
call denter
MakeStack

argpush 4,0 ;lpName (none)
argpush 3,1 ;bInitialState = true
argpush 2,1 ;bManualReset = true
argpush 1,0 ;lpEventAttributes
call [CreateEventW]

call dleave
ret
end if


;------------------------------------------------------------------------------
;This function will create an AsynchronousStruct for write operations with Asynchronous operations
;input:
;    1:pointer to AsynchronousStruct

;output:
;    EAX => pointer to AsynchronousStruct
;------------------------------------------------------------------------------
if used removeAsynchronousStruct
removeAsynchronousStruct:
push pesi

mov pesi,pdword [pesp+psize*2] ;AsynchronousStruct

xor eax,eax
xchg peax,pdword [pesi+(psize*2)+(4*2)] ;hEvent
push peax                               ;hEvent
call closeEvent

push pesi
call FreeMemLocation

pop pesi
ret psize
end if

;------------------------------------------------------------------------------
;This function sets the offset in an Asynchronous structure
;input:
;    1:pointer to AsynchronousStruct
;    2:x64: offset, x86:; pointer to offset   (use -1 for end of the file)

;output:
;    EAX => pointer to AsynchronousStruct
;------------------------------------------------------------------------------
if used setOffsetAsynchronousStruct
setOffsetAsynchronousStruct:
call denter

mov pedi,pdword [pebp+psize*2] ;OVERLAPPED
mov pesi,pdword [pebp+psize*3] ;x64: offset, x86:; pointer to offset

if used is64
  mov qword [pedi+(psize*2)],pesi
else
  add pedi,psize*2
  lodsd  ;load offset
  stosd  ;store offset
  lodsd  ;load offsetHigh
  stosd  ;store offsetHigh
end if

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function gets the offset of the Asynchronous structure
;input:
;       1:pointer to AsynchronousStruct

;output:
; (x64) RAX => offset
; (x86) RAX => pointer to offset
;------------------------------------------------------------------------------
if used getOffsetAsynchronousStruct
getOffsetAsynchronousStruct:
call denter

mov pesi,pdword [pebp+psize*2] ;OVERLAPPED

if used is64
  mov rax,qword [rsi+(psize*2)] ;offset
else
  lea eax, dword [esi+psize*2] ;offset
end if

call dleave

ret psize
end if

;------------------------------------------------------------------------------
;This function will wait for AsynchronousStruct to finish or for the time to eplapse
;input:
;    1:the number of AsynchronousStructs in the array
;    2:array of pointers to AsynchronousStruct
;    3:time to wait in milliseconds

;output:
;    EAX =>
;        0 => not finished yet
;        1 => finished
;------------------------------------------------------------------------------
if used WaitForAsynchronousStructs
WaitForAsynchronousStructs:
call denter

mov pecx,pdword [pebp+psize*2]  ;the number of AsynchronousStructs in the array
mov pesi,pdword [pebp+psize*3] ;array of pointers to AsynchronousStruct

loopThroughAsynchronousStructs:
  lodsp ;R/EAX => pointer to AsynchronousStruct
  push pdword [peax+(psize*2)+(4*2)]  ;hEvent
loop loopThroughAsynchronousStructs

mov pesi,pesp ;array with events
mov pecx,pdword [pebp+psize*2]  ;the number of AsynchronousStructs in the array


MakeStack
argpush 4,pdword [pebp+psize*4] ;dwMilliseconds
argpush 3,1                     ;bWaitAll
argpush 2,pesi                  ;array with events
argpush 1,pecx                  ;the number of events
call [WaitForMultipleObjects]
xor ecx,ecx
cmp peax,0x00000102 ;WAIT_TIMEOUT
je endWaitForAsynchronousStructs

inc ecx

endWaitForAsynchronousStructs:
mov eax,ecx
call dleave
ret psize*3
end if

;------------------------------------------------------------------------------
;This function will return the written amount of bytes by the Asynchronous write or read function
;input:
;    1:file handle
;    2:pointers to AsynchronousStruct

;output:
;    EAX => amount of bytes transfered
;------------------------------------------------------------------------------
if used GetWrittenBytesAsynchronous
GetWrittenBytesAsynchronous:
call denter
MakeStack

if used is32
  push 0 ;LPDWORD
  mov eax,esp
end if

argpush 4,1                     ;bWait
if used is64
  lea r8,qword[rsp+psize*5]     ;NumberOfBytesTransferred
else
  push eax
end if
argpush 2,pdword [pebp+psize*3] ;Overlapped
argpush 1,pdword [pebp+psize*2] ;file handle
call [GetOverlappedResult]

if used is64
  mov rax,qword[rsp+psize*5]     ;NumberOfBytesTransferred
else
  pop eax                        ;NumberOfBytesTransferred
end if

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will set the file pointer to desired position
;input:
;    1:file handle
;    2:(x64)filepointer to seek to (x86)pointer to a long long with a filepointer to seek to

;output:
;    EAX => return value SetFilePointer (the low-order dword of the current file pointer
;------------------------------------------------------------------------------
if used seekToPointerFile
seekToPointerFile:
call denter
MakeStack

mov peax,pdword [pebp+psize*3] ;(x64)pointer to seek to (x86)pointer to a long long to seek to

if used is32
  ;SetFilePointerEx is avaidable since win 2000, so call SetFilePointer in 32-bit to support winNT
  push FILE_BEGIN            ;seek from begining
  push dword [peax+psize]    ;high part of the long long
  push dword [peax]          ;low part of the long long
  push dword [pebp+psize*2]  ;file handle
  call [SetFilePointer]
else
  ;x64 is avaidable since Win XP, so we are able to use SetFilePointerEx
  argpush 4,FILE_BEGIN            ;seek from begining
  argpush 3,0                     ;high part of the long long
  argpush 2,rax                   ;long long
  argpush 1,pdword [pebp+psize*2] ;file handle
  call [SetFilePointerEx]
end if


call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will set the file pointer to the end of the file
;input:
;    1:file handle

;output:
;    EAX => return value SetFilePointer (the low-order dword of the current file pointer
;------------------------------------------------------------------------------
if used seekToEndPointerFile
seekToEndPointerFile:
call denter
MakeStack

argpush 4,FILE_END              ;seek from begining
argpush 3,0                     ;high part of the long long
argpush 2,0                     ;low part of the long long
argpush 1,pdword [pebp+psize*2] ;file handle
call [SetFilePointer]

call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function creates an event that can be used by Asynchronous operations
;input:
;    1:pointer to event

;output:
;    EAX => event handle
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;This function will close a file handle
;input:
;    1:file handle

;output:
;    EAX => file handle
;------------------------------------------------------------------------------
if used closefile | used closeEvent
closeEvent:
closefile:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]
call [CloseHandle]

mov peax,pdword [pebp+psize*2]

call dleave
ret psize
end if


;------------------------------------------------------------------------------
;This function will change the last write time
;input:
;    1:file handle
;    2:x64:FILETIME x86:poiner to FILETIME

;output:
;    (R/E)AX => If the function succeeds, the return value is nonzero

;------------------------------------------------------------------------------
if used setRawLastWriteTime
setRawLastWriteTime:
call denter
MakeStack

if used is32
  push pdword [pebp+psize*3]
else
  lea r9,pdword [pebp+psize*3]
end if
argpush 3,0
argpush 2,0
argpush 1,pdword [pebp+psize*2]
call [SetFileTime]

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will return the last write time
;input x86:
;    1:file handle
;    2:FILETIME

;input x64:
;    1:file handle

;output x86:
;    EAX => boolean of success

;output x64:
;    EAX => zero or last writetime
;------------------------------------------------------------------------------
if used getRawLastWriteTime
getRawLastWriteTime:
call denter
MakeStack

if used is32
  argpush 4,pdword [pebp+psize*3]
else
  lea r9,qword [rsp+0x8*4]
end if
argpush 3,0
argpush 2,0
argpush 1,pdword [pebp+psize*2]
call [GetFileTime]

if used is64
  test rax,rax
  jz endGetRawLastWriteTime

  mov rax,qword [rsp+0x8*4]
end if

endGetRawLastWriteTime:
call dleave
if used is32
  ret psize*2
else
  ret psize
end if
end if

;------------------------------------------------------------------------------
;This function will return the attributes
;input:
;    1:path

;output:
;       eax:Attributes
;------------------------------------------------------------------------------
if used getFileAttributes
getFileAttributes:
call denter
MakeStack

;add '\\?\' if nececerry
push pdword [pebp+psize*2]
call createLongPath
mov pebx,peax

argpush 1,pebx ;path
call [GetFileAttributesW]

cmp pebx,pdword [pebp+psize*2]
je dontFreePointer__getFileAttributes

  push peax ;save return value

  push pebx
  call FreeMemLocation

  pop peax

dontFreePointer__getFileAttributes:

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will set the fileattributes
;input:
;    1:path
;    2:attributes:
;       0x1 FILE_ATTRIBUTE_READONLY
;       0x2 FILE_ATTRIBUTE_HIDDEN
;       0x4 FILE_ATTRIBUTE_SYSTEM
;       0x20 FILE_ATTRIBUTE_ARCHIVE
;       0x80 FILE_ATTRIBUTE_NORMAL
;       0x100 FILE_ATTRIBUTE_TEMPORARY
;       0x1000 FILE_ATTRIBUTE_OFFLINE
;       0x4000 FILE_ATTRIBUTE_ENCRYPTED

;output:
;       eax:Attributes
;------------------------------------------------------------------------------
if used setFileAttributes
setFileAttributes:
call denter
MakeStack

;add '\\?\' if nececerry
push pdword [pebp+psize*2]
call createLongPath
mov pebx,peax

argpush 2,pdword [pebp+psize*3] ;attributes
argpush 1,pebx                  ;path
call [SetFileAttributesW]

cmp pebx,pdword [pebp+psize*2]
je dontFreePointer__setFileAttributes

  push peax ;save return value

  push pebx
  call FreeMemLocation

  pop peax

dontFreePointer__setFileAttributes:

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will always return a path that can exeet the 255 limit, no mather how long the path is
;It can still return the original pointer if the path is already a long path
;input:
;    1:path

;output:
;       R/Eax:either a new created string with \\?\ in front of it, or the same string in Path
;------------------------------------------------------------------------------
if used forceCreateLongPath
forceCreateLongPath:
call denter
mov pesi,pdword [pebp+psize*2] ;path
jmp __forceCreateLongPath
end if

;------------------------------------------------------------------------------
;This function create a path that can exeet the 255 limit or return the path if it is already aware
;input:
;    1:path

;output:
;       R/Eax:either a new created string with \\?\ in front of it, or the same string in Path
;------------------------------------------------------------------------------
if used createLongPath | used forceCreateLongPath
createLongPath:
call denter
mov pesi,pdword [pebp+psize*2] ;path

push pesi
call strlenW
shr peax,1 ;devide by 2
cmp peax,260
mov peax,pdword [pebp+psize*2] ;path
jb doesNotExeedMaximumChars

__forceCreateLongPath:
if used is32
  lodsd
  cmp eax,0x005c005c ;'\\'
  jne createLongPathNow
  lodsd
  cmp eax,0x005c003F ;'?\'
  mov peax,pdword [pebp+psize*2] ;path
  mov ebx,eax
  je fixPathBackwardSlash
else
  lodsq
  mov rdx,0x5C003F005C005C   ;'\\?\'
  cmp rax,rdx
  mov rax,pdword [pebp+psize*2] ;path
  mov rbx,rax
  je fixPathBackwardSlash
end if


createLongPathNow:
push 0
push pdword [pebp+psize*2] ;path
push 0x2
call pushLongPathsAllowed

  db '\\?\',0

pushLongPathsAllowed:
push 0x1
call CreateOneString

mov pebx,peax ;save return string

fixPathBackwardSlash:
mov pesi,peax
mov pedi,pesi

;long paths only accept '\', not '/'
changeForwardToBackwardSlash:
  lodsw
  cmp ax,'/'
  jne nextChar

    mov ax,'\'

  nextChar:
  stosw
  test ax,ax
jnz changeForwardToBackwardSlash

mov peax,pebx ;restor return string

doesNotExeedMaximumChars:
endCreateLongPath:
call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function replaces '/' with '\' and returns if the path ens with an '\'
;input:
;    1:path (UTF-8)

;output:
;       R/Eax:True = end with '\', false if not
;------------------------------------------------------------------------------
if used fixPath_Utf8
fixPath_Utf8:
call denter

mov pesi,pdword [pebp+psize*2] ;Path UTF-8
mov pedi,pesi

;Windows paths use '\', not '/'

xor eax,eax 
changeForwardToBackwardSlash_fixPath:
  mov cl,al ;save last unicode point
  lodsb

  ;replace '\' with '\'
  cmp al,'/'
  jne checkDubbleBackslah

    mov al,'\'


    ;remove dubble backslash
    cmp cl,'\'
    je changeForwardToBackwardSlash_fixPath

  checkDubbleBackslah:
  cmp al,'\'
  jne nextChar__utf8

    ;remove dubble backslash
    cmp cl,'\'
    je changeForwardToBackwardSlash_fixPath

  nextChar__utf8:
  stosb

  test al,al
jnz changeForwardToBackwardSlash_fixPath

cmp cl,'\'
jne endFixPath

  inc eax

endFixPath:
call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will check if a directorie exist
;input:
;    1:path

;output:
;       eax:0 (no directoty) 0x10:directory
;flags: Z: No directory  z: directory
;------------------------------------------------------------------------------
if used isDirectory
isDirectory:
call denter
MakeStack

;add '\\?\' if nececerry
push pdword [pebp+psize*2]
call createLongPath
mov pebx,peax

argpush 1,pebx ;path
call [GetFileAttributesW]

cmp pebx,pdword [pebp+psize*2]
je dontFreePointer

  push peax ;save return value

  push pebx
  call FreeMemLocation

  pop peax

dontFreePointer:
call dleave

cmp eax,-1
jz setEAXZero

and eax,FILE_ATTRIBUTE_DIRECTORY

endIsDirectory:
ret psize

setEAXZero:
xor eax,eax
jmp endIsDirectory
end if

;------------------------------------------------------------------------------
;This function will create a directory
;input:
;    1:path

;output:
;       eax:0 error, NOT ZERO:success
;flags: Z: Error  z: success
;------------------------------------------------------------------------------
if used createDirectory
createDirectory:
call denter
MakeStack

;add '\\?\' if nececerry
push pdword [pebp+psize*2]
call createLongPath
mov pebx,peax

argpush 2,0    ;lpSecurityAttributes
argpush 1,pebx ;path
call [CreateDirectoryW]

cmp pebx,pdword [pebp+psize*2]
je dontFreePointer__createDirectory

  push peax ;save return value

  push pebx
  call FreeMemLocation

  pop peax

dontFreePointer__createDirectory:

call dleave

test peax,peax
ret psize
end if

;------------------------------------------------------------------------------
;This function start (and return an handle) to enumerate file(s) and directories in a directory
;input:
;    1:path
;    2: return structure (total: 4+(7*2)+4+8+260=290, rounded by 8=296):
;       Atributes (dword):
;           0x10 Directory
;       TIMEDATE structure (7*2 + 4)
;       Filesize (long long, no pointer)
;       Filename[260]



;output:
;       eax:HANDLE or zero
;------------------------------------------------------------------------------
if used startDirectoryEnumeration
directoryWildFlag db '*.*',0

startDirectoryEnumeration:
call denter
sub pesp,11*4 + 264 + 16 + 4 ;create an _WIN32_FIND_DATA on stack (+4 for rouning up)
mov pesi,pesp ;(R/E)SI => pointer to _WIN32_FIND_DATA structure

;zero structure
mov pedi,pesp ;pointer to _WIN32_FIND_DATA structure
xor eax,eax
if used is32
  mov ecx,82
  rep stosd
else
  mov ecx,41
  rep stosq
end if

MakeStack

push 0 ;flags
push directoryWildFlag
push pdword [pebp+psize*2] ;path
call combineStrings
mov pedi,peax ;(R/E)SI => pointer to path + "*.*"

argpush 2,pesi     ;LPWIN32_FIND_DATA
argpush 1,pedi     ;path
call [FindFirstFileA]
mov pebx,peax  ;(R/E)BX => handle

;free temp path string
push pedi
call FreeMemLocation

inc pebx    ;-1 + 1 = 0
test pebx,pebx
cmovz eax,edx
jz endStartDirectoryEnumeration ;return if invalid handle
dec pebx  ;restore handle

mov pedi,pdword [pebp+psize*3] ;return structure

movsd ;copy atributes
add pesi,8*2 ;skip ftCreationTime and LastAccessTime

stosw ;skip first two bytes

;convert last write time
argpush 2,pedi ;SYSTEMTIME structure
argpush 1,pesi ;last write time
call [FileTimeToSystemTime]

;convert year wordt to dword
xor ax,ax
xchg ax,word [pedi]
mov word [pedi-2],ax

add pesi,8 ;skip ftLastWriteTime
add pedi,(8*2) ;skip SYSTEMTIME structure

;swap high and low
lodsd ;get high dword of the size member
movsd ;copy the low dword of the size member
stosd ;store tthe high dword

add pesi,8 ;skip reserved values

if used is32
  mov ecx,65 ;=260 / 4
  rep movsd ;copy the filename
else
  mov ecx,32 ;=256 / 4
  rep movsq ;copy the first 256 bytes of the filename
  movsd     ;copy last 4 bytes
end if


mov peax,pebx ;return handle

endStartDirectoryEnumeration:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function return next next file in the directory
;input:
;    1:handle
;    2: return structure (total: 4+(7*2)+4+8+260=290, rounded by 8=296):
;       Atributes (dword):
;           0x10 Directory
;       TIMEDATE structure (7*2 + 4)
;       Filesize (long long, no pointer)
;       Filename[260]



;output:
;       eax:true of false
;------------------------------------------------------------------------------
if used nextDirectoryEnumeration
nextDirectoryEnumeration:
call denter
sub pesp,(11*4) + 264 + 16+ 4 ;create an _WIN32_FIND_DATA on stack (+4 for rouning up)
mov pesi,pesp ;pointer to _WIN32_FIND_DATA structure

;zero structure
mov pedi,pesp ;pointer to _WIN32_FIND_DATA structure
xor eax,eax
if used is32
  mov ecx,82
  rep stosd
else
  mov ecx,41
  rep stosq
end if

MakeStack

argpush 2,pesi                      ;LPWIN32_FIND_DATA
argpush 1,pdword [pebp+psize*2]     ;handle
call [FindNextFileA]

test peax,peax
jz endNextDirectoryEnumeration ;return if invalid handle

mov pedi,pdword [pebp+psize*3] ;return structure

movsd ;copy atributes
add pesi,8*2 ;skip ftCreationTime and LastAccessTime

stosw ;skip first two bytes

;convert last write time
argpush 2,pedi ;SYSTEMTIME structure
argpush 1,pesi ;last write time
call [FileTimeToSystemTime]

;convert year wordt to dword
xor ax,ax
xchg ax,word [pedi]
mov word [pedi-2],ax

add pesi,8 ;skip ftLastWriteTime
add pedi,(8*2) ;skip SYSTEMTIME structure

;swap high and low
lodsd ;get high dword of the size member
movsd ;copy the low dword of the size member
stosd ;store tthe high dword

add pesi,8 ;skip reserved values

if used is32
  mov ecx,65 ;=260 / 4
  rep movsd ;copy the filename
else
  mov ecx,32 ;=256 / 4
  rep movsq ;copy the first 256 bytes of the filename
  movsd     ;copy last 4 bytes
end if

xor eax,eax
inc eax

endNextDirectoryEnumeration:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function close the handle of the enumeration
;input:
;    1:handle


;output:
;       none
;------------------------------------------------------------------------------
if used closeDirectoryEnumeration
closeDirectoryEnumeration:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]
call [FindClose]

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function returns a pointer (allocated by GetMemLocation) with the directory
;input:
;    1:modhandle


;output:
;       Pointer with directory or zero
;------------------------------------------------------------------------------
if used getDirectoryByModule
getDirectoryByModule:
call denter
MakeStack

push 260
call GetMemLocation
mov pebx,peax

;call GetModuleFileName to receive the path to thie application
argpush 3,260
argpush 2,pebx
argpush 1,0
call [GetModuleFileNameA]
test peax,peax
jz failed
cmp peax,260
jz failed

;add a end the of filename and add a quote there
lea pedi,pdword [pebx+260]

xor ecx,ecx  ; zero ECX
dec pecx     ; (R/E)CX => -1  (scasb will be repeated for 0xFFFFFFFF(FFFFFFFF) times, or until zero byte is found)
mov eax,'\'  ; byte to find with scasb => '\\'

STD ;execute scasb function backward
repnz scasb   ;loop string until something else then a zero byte is found
CLD ;clear backward direction

;place a zero byte at the slash to get the executable direcotry
mov byte [pedi+2],0

endGetDirectoryByModule:
mov peax,pebx

call dleave
ret

failed:
push pebx
call FreeMemLocation

xor ebx,ebx
jmp endGetDirectoryByModule
end if

;------------------------------------------------------------------------------
;This function displays a window to the user where it can select a folder
;input:
;    1:information string (may be zero)


;output:
;       Pointer with directory or zero
;------------------------------------------------------------------------------

BIF_EDITBOX equ 0x00000010
BIF_NEWDIALOGSTYLE equ 0x00000040
BIF_NONEWFOLDERBUTTON equ 0x200

;1:title
if used BrouwseForFolder
BrouwseForFolder:
mov peax,BIF_NONEWFOLDERBUTTON ;flags
xchg pdword [pesp+psize*1],peax  ;title => flags
xchg pdword [pesp],peax  ;return address => title
push peax                ;push return address
end if

;1:title, 2:flags
if used BrouwseForFolderEx | used BrouwseForFolder
BrouwseForFolderEx:
call denter
xor pebx,pebx
sub pesp,(psize*4) + psize + (psize*3) ;create an BROWSEINFO on stack (+4 for rouning up)
mov pesi,pesp ;pointer to BROWSEINFO structure

;zero structure
mov pedi,pesp ;pointer to BROWSEINFO structure
xor eax,eax
if used is32
  mov ecx,8
  rep stosd
else
  mov ecx,8
  rep stosq
end if

cmp pdword [pebp+psize*2],0
je pushStandartString

push pdword [pebp+psize*2]
jmp pushCaptionOnStack

pushStandartString:
call pushCaptionOnStack

du 'Select folder',0

pushCaptionOnStack:

pop pdword [pesi+(psize*3)]
mov eax,dword [pebp+psize*3] ;flags
mov dword [pesi+(psize*4)],BIF_EDITBOX or BIF_NEWDIALOGSTYLE
or dword [pesi+(psize*4)],eax

MakeStack

argpush 2,0x2 ;COINIT_APARTMENTTHREADED
argpush 1,0
call [CoInitializeEx]

callBrouwseForFolderAgain:
argpush 1,pesi
call [SHBrowseForFolder]
test peax,peax
jz endBrouwseForFolder
mov pebx,peax

push 261*2
call GetMemLocation
xchg pebx,peax

argpush 2,pebx
argpush 1,peax
call [SHGetPathFromIDList]

push pebx
call strlenW
test peax,peax
jz callBrouwseForFolderAgain

mov peax,pebx
endBrouwseForFolder:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function converts an FILETIME structure to a TIMEDATE structure
;input:
;    1:FILETIME Qword (x64),  pointer to FILETIME structure (x86)
;    2:pointer to TIMEDATE
;
;output:
;       eax:return value of FileTimeToSystemTime
;------------------------------------------------------------------------------
if used getTimeDateFromFileTime
getTimeDateFromFileTime:
call denter
MakeStack

mov pedi,pdword [pebp+psize*3]
stosw  ;forward 2 bytes

;convert last write time
argpush 2,pedi             ;SYSTEMTIME structure
if used is32
  push dword [ebp+psize*2] ;last write time
else
  lea rcx,qword [rbp+psize*2] ;last write time
end if
call [FileTimeToSystemTime]

;convert year wordt to dword
xor ax,ax
xchg ax,word [pedi]
mov word [pedi-2],ax

call dleave
ret psize*2
end if

if used FindFirstFile_win
FindFirstFile_win:
if used is32
  jmp [FindFirstFile]
else
  call denter
  MakeStack

  argpush 2,pdword [pebp+(psize*3)]
  argpush 1,pdword [pebp+(psize*2)]
  call [FindFirstFile]

  call dleave
  ret psize*2
end if
end if

;------------------------

if used FindNextFile_win
FindNextFile_win:
if used is32
  jmp [FindNextFile]
else
  call denter
  MakeStack

  argpush 2,pdword [pebp+(psize*3)]
  argpush 1,pdword [pebp+(psize*2)]
  call [FindNextFile]

  call dleave
  ret psize*2
end if
end if


;------------------------
if used FindClose_win
FindClose_win:
if used is32
  jmp [FindClose]
else
  call denter
  MakeStack

  argpush 1,pdword [pebp+(psize*2)]
  call [FindClose]

  call dleave
  ret psize*1
end if
end if

if used FileAttributes & ~used NOFileAttributes
FileAttributes db 'READONLY',0,\
                  'HIDDEN',0,\
                  'SYSTEM',0,\
                  0,\
                  'DIRECTORY',0,\
                  'ARCHIVE',0,\
                  'DEVICE',0,\
                  'NORMAL',0,\
                  'TEMPORARY',0,\
                  'SPARSE_FILE',0,\
                  'REPARSE_POINT',0,\
                  'COMPRESSED',0,\
                  'OFFLINE',0,\
                  'NOT_CONTENT_INDEXED',0,\
                  'ENCRYPTED',0,\
                  'INTEGRITY_STREAM',0,\
                  0,\
                  'NO_SCRUB_DATA',0,\
                  'RECALL_ON_OPEN',0,\
                  'PINNED',0,\
                  'UNPINNED',0,\
                  0,\
                  'RECALL_ON_DATA_ACCESS',0,\
                  0,\
                  0,\
                  0,\
                  0,\
                  0,\
                  0,\
                  0,\
                  0,\
                  0
end if